/**
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <memory>
#include <utility>
#include <string>
#include <vector>

#include "PyInputStream.h"

#include "utils/gsl.h"

namespace org::apache::nifi::minifi::python {

PyInputStream::PyInputStream(std::shared_ptr<io::InputStream> stream)
    : stream_(std::move(stream)) {
}

py::bytes PyInputStream::read() {
  return read(stream_->size());
}

py::bytes PyInputStream::read(size_t len) {
  if (len == 0) {
    len = stream_->size();
  }

  if (len <= 0) {
    return nullptr;
  }

  std::vector<std::byte> buffer(len);

  const auto read = stream_->read(buffer);
  return {reinterpret_cast<char *>(buffer.data()), read};
}

}  // namespace org::apache::nifi::minifi::python
